#!/usr/bin/env python3
import boto3
import time
import json

print("[*] Creating DynamoDb tables")
dynamo_client = boto3.client('dynamodb', region_name="us-east-1", endpoint_url="https://localhost:4569",
                             aws_access_key_id="", aws_secret_access_key="", verify=False)




def create_kv_table_without_sort_key(tableName, hasTtl=False, keyName='partitionKey', sortName='sortKey'):
    print("    [+] Creating DynamoDB table %s" % tableName)
    dynamo_client.create_table(TableName=tableName,
                               KeySchema=[{'AttributeName': keyName, 'KeyType': 'HASH'}],
                               AttributeDefinitions=[{'AttributeName': keyName, 'AttributeType': 'S'}],
                               ProvisionedThroughput={'ReadCapacityUnits': 10, 'WriteCapacityUnits': 10})
    dynamo_client.get_waiter('table_exists').wait(TableName=tableName)
    if hasTtl:
        try:
            dynamo_client.update_time_to_live(TableName=tableName, TimeToLiveSpecification={
                'Enabled': True,
                'AttributeName': 'objectTTL'
            })
        except:
            pass  # Mock dynamoDb implementation doesn't support TTL presently

def create_kv_table(tableName, hasTtl=False, keyName='partitionKey', sortName='sortKey'):
    print("    [+] Creating DynamoDB table %s" % tableName)
    dynamo_client.create_table(TableName=tableName,
                               KeySchema=[{'AttributeName': keyName, 'KeyType': 'HASH'},
                                          {'AttributeName': sortName, 'KeyType': 'RANGE'}],
                               AttributeDefinitions=[{'AttributeName': keyName, 'AttributeType': 'S'},
                                                     {'AttributeName': sortName, 'AttributeType': 'S'}],
                               ProvisionedThroughput={'ReadCapacityUnits': 10, 'WriteCapacityUnits': 10})
    dynamo_client.get_waiter('table_exists').wait(TableName=tableName)
    if hasTtl:
        try:
            dynamo_client.update_time_to_live(TableName=tableName, TimeToLiveSpecification={
                'Enabled': True,
                'AttributeName': 'objectTTL'
            })
        except:
            pass  # Mock dynamoDb implementation doesn't support TTL presently


def create_stream_table(tableName):
    print("    [+] Creating DynamoDB  table %s for kinesis stream" % tableName)
    dynamo_client.create_table(TableName=tableName,
                               KeySchema=[{'AttributeName': 'leaseKey', 'KeyType': 'HASH'}],
                               AttributeDefinitions=[{'AttributeName': 'leaseKey', 'AttributeType': 'S'}],
                               ProvisionedThroughput={'ReadCapacityUnits': 100, 'WriteCapacityUnits': 100})
    dynamo_client.get_waiter('table_exists').wait(TableName=tableName)


def init_persistence_unit():
    create_kv_table(tableName='local_org_persistence_unit', keyName='orgKey', sortName='unitType')
    create_kv_table(tableName='local_persistence_unit', keyName='unitType', sortName='name')

    print("    [*] Populating persistence unit tables")
    dynamo_resource = boto3.resource('dynamodb', region_name="us-east-1", endpoint_url="https://localhost:4569",
                                     aws_access_key_id="", aws_secret_access_key="", verify=False)
    persistence_unit_table = dynamo_resource.Table('local_persistence_unit')
    persistence_unit_table.put_item(Item={
        "unitType": "EVENT_CLUSTER",
        "config": {
            "url": "elasticsearch:9300"
        },
        "name": "EVENT_00",
        "isCurrent": True
    })

    org_persistence_unit_table = dynamo_resource.Table('local_org_persistence_unit')

    # Org 1 = R76KWHRN
    # Org 2 = K7MGDHQ7
    for org_key in ['R76KWHRN', 'K7MGDHQ7']:
        org_persistence_unit_table.put_item(Item={
            "persistenceUnit": {
                "unitType": "EVENT_CLUSTER",
                "config": {
                    "url": "elasticsearch:9300"
                },
                "name": "EVENT_00",
                "isCurrent": True
            },
            "orgKey": org_key,
            "unitType": "EVENT_CLUSTER"
        })

def init_defense_configuration():
    create_kv_table_without_sort_key(tableName='local_defense_configuration', keyName='partitionKey')

def create_dynamo_tables():
    create_stream_table(tableName='develop-augmentation-kinesis-app')
    create_stream_table(tableName='develop-deviceevent-kinesis-app')

    create_kv_table(tableName='cbd-test-AugmentedBehaviorEvent', hasTtl=True)
    create_kv_table(tableName='reputationStorage', hasTtl=True)
    create_kv_table(tableName='cbd-test-DeviceAnalytics')
    create_kv_table(tableName='cbd-origin-event', hasTtl=True)
    create_kv_table(tableName='lq-results-summary', hasTtl=True)
    create_kv_table(tableName='lq-device-results-summary', hasTtl=True)
    create_kv_table(tableName='md5lookup', hasTtl=True)
    create_kv_table(tableName='threat', hasTtl=True)
    create_kv_table(tableName='orgThreat', hasTtl=True)
    create_kv_table(tableName='deviceanalytics', hasTtl=True)
    create_kv_table(tableName='originEventSource', hasTtl=True)
    init_persistence_unit()
    init_defense_configuration()


if __name__ == '__main__':
    create_dynamo_tables()
